unit ZIPTree;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, AwkMain, ZipTV;

type
  TZIPTree = class(TTreeView)
  private
    FZipTV: TZipTV;
    FFiles:TStringList;
    FImageList:TImageList;
    FShowFiles: boolean;
    procedure SetZipTV(const Value: TZipTV);
    procedure SetShowFiles(const Value: boolean);
    { Private declarations }
  protected
    { Protected declarations }
    function HasDelimiter(Str:string):boolean;
    function GetLevel(Str:string):integer;
    function IsFolder(Str:string):boolean;
    function GetNode(Str:string):TTreeNode;
    function ExtractNode(Str:string;Level:integer):string;
    function AddFile(Node:TTreeNode;Str:string):TTreeNode;
    function TrimFirst(Str:string):string;
    function GetFirst(Str:string):string;
    function NodeExists(Str:string):boolean;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure UpdateTree;
  published
    { Published declarations }
    property ZipTV:TZipTV read FZipTV write SetZipTV;
    property ShowFiles:boolean read FShowFiles write SetShowFiles;
  end;

procedure Register;

implementation

{$R ZipTree.res}

procedure Register;
begin
  RegisterComponents('System', [TZIPTree]);
end;

{ TZIPTree }

constructor TZIPTree.Create(AOwner: TComponent);
var BMP:TBitmap;
begin
  inherited Create(AOwner);
  FFiles:=TStringList.Create;
  FImageList:=TImageList.CreateSize(16,16);
  BMP:=TBitmap.Create;
  BMP.LoadFromResourceName(HInstance,'FOLDER');
  BMP.Width:=16; BMP.Height:=16;
  FImageList.AddMasked(BMP,clWhite);
  BMP.Free;
  BMP:=TBitmap.Create;
  BMP.Mask(clWhite);
  BMP.LoadFromResourceName(HInstance,'OPENFOLDER');
  BMP.Width:=16; BMP.Height:=16;
  FImageList.AddMasked(BMP,clBlue);
  BMP.Free;
  Images:=FImageList;
end;

destructor TZIPTree.Destroy;
begin
  FFiles.Free;
  FImageList.Free;
  inherited Destroy;
end;

procedure TZIPTree.SetZipTV(const Value: TZipTV);
begin
  FZipTV := Value;
end;

procedure TZIPTree.UpdateTree;
var i,j,level:integer;
    Node,RootNode:TTreeNode;
begin
Screen.Cursor:=crAppStart;
  if Assigned(FZipTV) then
     if FZipTV.ArchiveFile<>'' then
        FZipTV.FilesInArchive(FFiles,'*.*')
     else exit
  else exit;
Items.Clear;
RootNode:=Items.Add(nil,ExtractFileName(FZipTV.ArchiveFile));
level:=0;
for i:=0 to FFiles.Count-1 do
    if GetLevel(FFiles[i])>level then
       level:=GetLevel(FFiles[i]);
for i:=0 to FFiles.Count-1 do
    begin
    if GetLevel(FFiles[i])=0 then
       begin
       if not ShowFiles then
          if not IsFolder(FFiles[i]) then
             if not HasDelimiter(FFiles[i]) then exit;
       Node:=Items.AddChild(RootNode,FFiles[i]);
       Node.ImageIndex:=0;
       Node.SelectedIndex:=1;
       end
    else
        begin
        AddFile(RootNode,FFiles[i]);
        end;
    end;
Screen.Cursor:=crDefault;    
end;

function TZIPTree.GetLevel(Str: string): integer;
var i:integer;
    s:string;
begin
if not HasDelimiter(str) then
   begin
   Result:=0;
   exit;
   end;
i:=0;
s:=str;
while pos('\',s)>0 do
      begin
      inc(i);
      s:=copy(s,pos('\',s)+1,Length(s));
      end;
Result:=i;
end;

function TZIPTree.HasDelimiter(Str: string): boolean;
begin
if Pos('\',Str)>0 then Result:=true else Result:=false;
end;

function TZIPTree.IsFolder(Str: string): boolean;
begin
Result:=false;
if FZipTV.GetFileInfo(Str) then
   Result:=FZipTV.FileAttribute.ftDirectory;
end;

function TZIPTree.GetNode(Str: string): TTreeNode;
var i:integer;
begin
result:=nil;
for i:=0 to Items.Count-1 do
    begin
    if ANSIUpperCase(Str)=ANSIUpperCase(Items.Item[i].Text) then
       begin
       Result:=Items.Item[i];
       end;
    end;
end;

function TZIPTree.ExtractNode(Str: string; Level: integer): string;
var i,c:integer;
begin
c:=0;
Result:='';
for i:=1 to Length(Str) do
    begin
    if Str[i]='\' then
       begin
       INC(c);
       continue;
       end;
    if c=level then Result:=Result+Str[i];
    end;
end;

function TZIPTree.AddFile(Node: TTreeNode; Str: string): TTreeNode;
var i:integer;
    nd:TTreeNode;
begin
if GetLevel(Str)=0 then
   begin
   if not IsFolder(Str) then
      if not ShowFiles then exit;
   nd:=Items.AddChild(Node,Str);
   Nd.ImageIndex:=-1;
   Nd.SelectedIndex:=-1;
   exit;
   end
else
   begin
   if NodeExists(GetFirst(Str)) then
      begin
      AddFile(GetNode(GetFirst(Str)),TrimFirst(Str));
      end
      else
          begin
          nd:=Items.AddChild(Node,GetFirst(Str));
          Nd.ImageIndex:=0;
          Nd.SelectedIndex:=0;
          AddFile(nd,TrimFirst(Str));
          end;
   end;
end;

function TZIPTree.TrimFirst(Str: string): string;
var i,c:integer;
begin
c:=0;
Result:='';
for i:=1 to Length(Str) do
    begin
    if Str[i]='\' then
       begin
       INC(c);
       end;
    if c>0 then Result:=Result+Str[i];
    end;
if Length(Result)>1 then
   Result:=Copy(Result,2,Length(Result)-1);
end;

function TZIPTree.GetFirst(Str: string): string;
var i:integer;
begin
Result:='';
for i:=1 to Length(Str) do
    begin
    if Str[i]='\' then exit
       else
    Result:=Result+Str[i];
    end;
end;

function TZIPTree.NodeExists(Str: string): boolean;
var i:integer;
begin
Result:=false;
for i:=0 to Items.Count-1 do
    begin
    if ANSIUpperCase(Str)=ANSIUpperCase(Items.Item[i].Text) then
       begin
       Result:=true;
       exit;
       end;
    end;
end;

procedure TZIPTree.SetShowFiles(const Value: boolean);
begin
  FShowFiles := Value;
end;

end.
